# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities

__all__ = ['SubnetPoolArgs', 'SubnetPool']

@pulumi.input_type
class SubnetPoolArgs:
    def __init__(__self__, *,
                 prefixes: pulumi.Input[Sequence[pulumi.Input[_builtins.str]]],
                 address_scope_id: Optional[pulumi.Input[_builtins.str]] = None,
                 default_prefixlen: Optional[pulumi.Input[_builtins.int]] = None,
                 default_quota: Optional[pulumi.Input[_builtins.int]] = None,
                 description: Optional[pulumi.Input[_builtins.str]] = None,
                 ip_version: Optional[pulumi.Input[_builtins.int]] = None,
                 is_default: Optional[pulumi.Input[_builtins.bool]] = None,
                 max_prefixlen: Optional[pulumi.Input[_builtins.int]] = None,
                 min_prefixlen: Optional[pulumi.Input[_builtins.int]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 project_id: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 shared: Optional[pulumi.Input[_builtins.bool]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 value_specs: Optional[pulumi.Input[Mapping[str, pulumi.Input[_builtins.str]]]] = None):
        """
        The set of arguments for constructing a SubnetPool resource.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] prefixes: A list of subnet prefixes to assign to the subnetpool.
               Neutron API merges adjacent prefixes and treats them as a single prefix. Each
               subnet prefix must be unique among all subnet prefixes in all subnetpools that
               are associated with the address scope. Changing this updates the prefixes list
               of the existing subnetpool.
        :param pulumi.Input[_builtins.str] address_scope_id: The Neutron address scope to assign to the
               subnetpool. Changing this updates the address scope id of the existing
               subnetpool.
        :param pulumi.Input[_builtins.int] default_prefixlen: The size of the prefix to allocate when the cidr
               or prefixlen attributes are omitted when you create the subnet. Defaults to the
               MinPrefixLen. Changing this updates the default prefixlen of the existing
               subnetpool.
        :param pulumi.Input[_builtins.int] default_quota: The per-project quota on the prefix space that can be
               allocated from the subnetpool for project subnets. Changing this updates the
               default quota of the existing subnetpool.
        :param pulumi.Input[_builtins.str] description: The human-readable description for the subnetpool.
               Changing this updates the description of the existing subnetpool.
        :param pulumi.Input[_builtins.int] ip_version: The IP protocol version.
        :param pulumi.Input[_builtins.bool] is_default: Indicates whether the subnetpool is default
               subnetpool or not. Changing this updates the default status of the existing
               subnetpool.
        :param pulumi.Input[_builtins.int] max_prefixlen: The maximum prefix size that can be allocated from
               the subnetpool. For IPv4 subnetpools, default is 32. For IPv6 subnetpools,
               default is 128. Changing this updates the max prefixlen of the existing
               subnetpool.
        :param pulumi.Input[_builtins.int] min_prefixlen: The smallest prefix that can be allocated from a
               subnetpool. For IPv4 subnetpools, default is 8. For IPv6 subnetpools, default
               is 64. Changing this updates the min prefixlen of the existing subnetpool.
        :param pulumi.Input[_builtins.str] name: The name of the subnetpool. Changing this updates the name of
               the existing subnetpool.
        :param pulumi.Input[_builtins.str] project_id: The owner of the subnetpool. Required if admin wants to
               create a subnetpool for another project. Changing this creates a new subnetpool.
        :param pulumi.Input[_builtins.str] region: The region in which to obtain the V2 Networking client.
               A Networking client is needed to create a Neutron subnetpool. If omitted, the
               `region` argument of the provider is used. Changing this creates a new
               subnetpool.
        :param pulumi.Input[_builtins.bool] shared: Indicates whether this subnetpool is shared across
               all projects. Changing this updates the shared status of the existing
               subnetpool.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: A set of string tags for the subnetpool.
        :param pulumi.Input[Mapping[str, pulumi.Input[_builtins.str]]] value_specs: Map of additional options.
        """
        pulumi.set(__self__, "prefixes", prefixes)
        if address_scope_id is not None:
            pulumi.set(__self__, "address_scope_id", address_scope_id)
        if default_prefixlen is not None:
            pulumi.set(__self__, "default_prefixlen", default_prefixlen)
        if default_quota is not None:
            pulumi.set(__self__, "default_quota", default_quota)
        if description is not None:
            pulumi.set(__self__, "description", description)
        if ip_version is not None:
            pulumi.set(__self__, "ip_version", ip_version)
        if is_default is not None:
            pulumi.set(__self__, "is_default", is_default)
        if max_prefixlen is not None:
            pulumi.set(__self__, "max_prefixlen", max_prefixlen)
        if min_prefixlen is not None:
            pulumi.set(__self__, "min_prefixlen", min_prefixlen)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if project_id is not None:
            pulumi.set(__self__, "project_id", project_id)
        if region is not None:
            pulumi.set(__self__, "region", region)
        if shared is not None:
            pulumi.set(__self__, "shared", shared)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)
        if value_specs is not None:
            pulumi.set(__self__, "value_specs", value_specs)

    @_builtins.property
    @pulumi.getter
    def prefixes(self) -> pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]:
        """
        A list of subnet prefixes to assign to the subnetpool.
        Neutron API merges adjacent prefixes and treats them as a single prefix. Each
        subnet prefix must be unique among all subnet prefixes in all subnetpools that
        are associated with the address scope. Changing this updates the prefixes list
        of the existing subnetpool.
        """
        return pulumi.get(self, "prefixes")

    @prefixes.setter
    def prefixes(self, value: pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]):
        pulumi.set(self, "prefixes", value)

    @_builtins.property
    @pulumi.getter(name="addressScopeId")
    def address_scope_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The Neutron address scope to assign to the
        subnetpool. Changing this updates the address scope id of the existing
        subnetpool.
        """
        return pulumi.get(self, "address_scope_id")

    @address_scope_id.setter
    def address_scope_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "address_scope_id", value)

    @_builtins.property
    @pulumi.getter(name="defaultPrefixlen")
    def default_prefixlen(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        The size of the prefix to allocate when the cidr
        or prefixlen attributes are omitted when you create the subnet. Defaults to the
        MinPrefixLen. Changing this updates the default prefixlen of the existing
        subnetpool.
        """
        return pulumi.get(self, "default_prefixlen")

    @default_prefixlen.setter
    def default_prefixlen(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "default_prefixlen", value)

    @_builtins.property
    @pulumi.getter(name="defaultQuota")
    def default_quota(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        The per-project quota on the prefix space that can be
        allocated from the subnetpool for project subnets. Changing this updates the
        default quota of the existing subnetpool.
        """
        return pulumi.get(self, "default_quota")

    @default_quota.setter
    def default_quota(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "default_quota", value)

    @_builtins.property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The human-readable description for the subnetpool.
        Changing this updates the description of the existing subnetpool.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "description", value)

    @_builtins.property
    @pulumi.getter(name="ipVersion")
    def ip_version(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        The IP protocol version.
        """
        return pulumi.get(self, "ip_version")

    @ip_version.setter
    def ip_version(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "ip_version", value)

    @_builtins.property
    @pulumi.getter(name="isDefault")
    def is_default(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        Indicates whether the subnetpool is default
        subnetpool or not. Changing this updates the default status of the existing
        subnetpool.
        """
        return pulumi.get(self, "is_default")

    @is_default.setter
    def is_default(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "is_default", value)

    @_builtins.property
    @pulumi.getter(name="maxPrefixlen")
    def max_prefixlen(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        The maximum prefix size that can be allocated from
        the subnetpool. For IPv4 subnetpools, default is 32. For IPv6 subnetpools,
        default is 128. Changing this updates the max prefixlen of the existing
        subnetpool.
        """
        return pulumi.get(self, "max_prefixlen")

    @max_prefixlen.setter
    def max_prefixlen(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "max_prefixlen", value)

    @_builtins.property
    @pulumi.getter(name="minPrefixlen")
    def min_prefixlen(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        The smallest prefix that can be allocated from a
        subnetpool. For IPv4 subnetpools, default is 8. For IPv6 subnetpools, default
        is 64. Changing this updates the min prefixlen of the existing subnetpool.
        """
        return pulumi.get(self, "min_prefixlen")

    @min_prefixlen.setter
    def min_prefixlen(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "min_prefixlen", value)

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The name of the subnetpool. Changing this updates the name of
        the existing subnetpool.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "name", value)

    @_builtins.property
    @pulumi.getter(name="projectId")
    def project_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The owner of the subnetpool. Required if admin wants to
        create a subnetpool for another project. Changing this creates a new subnetpool.
        """
        return pulumi.get(self, "project_id")

    @project_id.setter
    def project_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "project_id", value)

    @_builtins.property
    @pulumi.getter
    def region(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The region in which to obtain the V2 Networking client.
        A Networking client is needed to create a Neutron subnetpool. If omitted, the
        `region` argument of the provider is used. Changing this creates a new
        subnetpool.
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "region", value)

    @_builtins.property
    @pulumi.getter
    def shared(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        Indicates whether this subnetpool is shared across
        all projects. Changing this updates the shared status of the existing
        subnetpool.
        """
        return pulumi.get(self, "shared")

    @shared.setter
    def shared(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "shared", value)

    @_builtins.property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        A set of string tags for the subnetpool.
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "tags", value)

    @_builtins.property
    @pulumi.getter(name="valueSpecs")
    def value_specs(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[_builtins.str]]]]:
        """
        Map of additional options.
        """
        return pulumi.get(self, "value_specs")

    @value_specs.setter
    def value_specs(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "value_specs", value)


@pulumi.input_type
class _SubnetPoolState:
    def __init__(__self__, *,
                 address_scope_id: Optional[pulumi.Input[_builtins.str]] = None,
                 all_tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 created_at: Optional[pulumi.Input[_builtins.str]] = None,
                 default_prefixlen: Optional[pulumi.Input[_builtins.int]] = None,
                 default_quota: Optional[pulumi.Input[_builtins.int]] = None,
                 description: Optional[pulumi.Input[_builtins.str]] = None,
                 ip_version: Optional[pulumi.Input[_builtins.int]] = None,
                 is_default: Optional[pulumi.Input[_builtins.bool]] = None,
                 max_prefixlen: Optional[pulumi.Input[_builtins.int]] = None,
                 min_prefixlen: Optional[pulumi.Input[_builtins.int]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 prefixes: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 project_id: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 revision_number: Optional[pulumi.Input[_builtins.int]] = None,
                 shared: Optional[pulumi.Input[_builtins.bool]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 updated_at: Optional[pulumi.Input[_builtins.str]] = None,
                 value_specs: Optional[pulumi.Input[Mapping[str, pulumi.Input[_builtins.str]]]] = None):
        """
        Input properties used for looking up and filtering SubnetPool resources.
        :param pulumi.Input[_builtins.str] address_scope_id: The Neutron address scope to assign to the
               subnetpool. Changing this updates the address scope id of the existing
               subnetpool.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] all_tags: The collection of tags assigned on the subnetpool, which have been
               explicitly and implicitly added.
        :param pulumi.Input[_builtins.str] created_at: The time at which subnetpool was created.
        :param pulumi.Input[_builtins.int] default_prefixlen: The size of the prefix to allocate when the cidr
               or prefixlen attributes are omitted when you create the subnet. Defaults to the
               MinPrefixLen. Changing this updates the default prefixlen of the existing
               subnetpool.
        :param pulumi.Input[_builtins.int] default_quota: The per-project quota on the prefix space that can be
               allocated from the subnetpool for project subnets. Changing this updates the
               default quota of the existing subnetpool.
        :param pulumi.Input[_builtins.str] description: The human-readable description for the subnetpool.
               Changing this updates the description of the existing subnetpool.
        :param pulumi.Input[_builtins.int] ip_version: The IP protocol version.
        :param pulumi.Input[_builtins.bool] is_default: Indicates whether the subnetpool is default
               subnetpool or not. Changing this updates the default status of the existing
               subnetpool.
        :param pulumi.Input[_builtins.int] max_prefixlen: The maximum prefix size that can be allocated from
               the subnetpool. For IPv4 subnetpools, default is 32. For IPv6 subnetpools,
               default is 128. Changing this updates the max prefixlen of the existing
               subnetpool.
        :param pulumi.Input[_builtins.int] min_prefixlen: The smallest prefix that can be allocated from a
               subnetpool. For IPv4 subnetpools, default is 8. For IPv6 subnetpools, default
               is 64. Changing this updates the min prefixlen of the existing subnetpool.
        :param pulumi.Input[_builtins.str] name: The name of the subnetpool. Changing this updates the name of
               the existing subnetpool.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] prefixes: A list of subnet prefixes to assign to the subnetpool.
               Neutron API merges adjacent prefixes and treats them as a single prefix. Each
               subnet prefix must be unique among all subnet prefixes in all subnetpools that
               are associated with the address scope. Changing this updates the prefixes list
               of the existing subnetpool.
        :param pulumi.Input[_builtins.str] project_id: The owner of the subnetpool. Required if admin wants to
               create a subnetpool for another project. Changing this creates a new subnetpool.
        :param pulumi.Input[_builtins.str] region: The region in which to obtain the V2 Networking client.
               A Networking client is needed to create a Neutron subnetpool. If omitted, the
               `region` argument of the provider is used. Changing this creates a new
               subnetpool.
        :param pulumi.Input[_builtins.int] revision_number: The revision number of the subnetpool.
        :param pulumi.Input[_builtins.bool] shared: Indicates whether this subnetpool is shared across
               all projects. Changing this updates the shared status of the existing
               subnetpool.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: A set of string tags for the subnetpool.
        :param pulumi.Input[_builtins.str] updated_at: The time at which subnetpool was created.
        :param pulumi.Input[Mapping[str, pulumi.Input[_builtins.str]]] value_specs: Map of additional options.
        """
        if address_scope_id is not None:
            pulumi.set(__self__, "address_scope_id", address_scope_id)
        if all_tags is not None:
            pulumi.set(__self__, "all_tags", all_tags)
        if created_at is not None:
            pulumi.set(__self__, "created_at", created_at)
        if default_prefixlen is not None:
            pulumi.set(__self__, "default_prefixlen", default_prefixlen)
        if default_quota is not None:
            pulumi.set(__self__, "default_quota", default_quota)
        if description is not None:
            pulumi.set(__self__, "description", description)
        if ip_version is not None:
            pulumi.set(__self__, "ip_version", ip_version)
        if is_default is not None:
            pulumi.set(__self__, "is_default", is_default)
        if max_prefixlen is not None:
            pulumi.set(__self__, "max_prefixlen", max_prefixlen)
        if min_prefixlen is not None:
            pulumi.set(__self__, "min_prefixlen", min_prefixlen)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if prefixes is not None:
            pulumi.set(__self__, "prefixes", prefixes)
        if project_id is not None:
            pulumi.set(__self__, "project_id", project_id)
        if region is not None:
            pulumi.set(__self__, "region", region)
        if revision_number is not None:
            pulumi.set(__self__, "revision_number", revision_number)
        if shared is not None:
            pulumi.set(__self__, "shared", shared)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)
        if updated_at is not None:
            pulumi.set(__self__, "updated_at", updated_at)
        if value_specs is not None:
            pulumi.set(__self__, "value_specs", value_specs)

    @_builtins.property
    @pulumi.getter(name="addressScopeId")
    def address_scope_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The Neutron address scope to assign to the
        subnetpool. Changing this updates the address scope id of the existing
        subnetpool.
        """
        return pulumi.get(self, "address_scope_id")

    @address_scope_id.setter
    def address_scope_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "address_scope_id", value)

    @_builtins.property
    @pulumi.getter(name="allTags")
    def all_tags(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        The collection of tags assigned on the subnetpool, which have been
        explicitly and implicitly added.
        """
        return pulumi.get(self, "all_tags")

    @all_tags.setter
    def all_tags(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "all_tags", value)

    @_builtins.property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The time at which subnetpool was created.
        """
        return pulumi.get(self, "created_at")

    @created_at.setter
    def created_at(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "created_at", value)

    @_builtins.property
    @pulumi.getter(name="defaultPrefixlen")
    def default_prefixlen(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        The size of the prefix to allocate when the cidr
        or prefixlen attributes are omitted when you create the subnet. Defaults to the
        MinPrefixLen. Changing this updates the default prefixlen of the existing
        subnetpool.
        """
        return pulumi.get(self, "default_prefixlen")

    @default_prefixlen.setter
    def default_prefixlen(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "default_prefixlen", value)

    @_builtins.property
    @pulumi.getter(name="defaultQuota")
    def default_quota(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        The per-project quota on the prefix space that can be
        allocated from the subnetpool for project subnets. Changing this updates the
        default quota of the existing subnetpool.
        """
        return pulumi.get(self, "default_quota")

    @default_quota.setter
    def default_quota(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "default_quota", value)

    @_builtins.property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The human-readable description for the subnetpool.
        Changing this updates the description of the existing subnetpool.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "description", value)

    @_builtins.property
    @pulumi.getter(name="ipVersion")
    def ip_version(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        The IP protocol version.
        """
        return pulumi.get(self, "ip_version")

    @ip_version.setter
    def ip_version(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "ip_version", value)

    @_builtins.property
    @pulumi.getter(name="isDefault")
    def is_default(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        Indicates whether the subnetpool is default
        subnetpool or not. Changing this updates the default status of the existing
        subnetpool.
        """
        return pulumi.get(self, "is_default")

    @is_default.setter
    def is_default(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "is_default", value)

    @_builtins.property
    @pulumi.getter(name="maxPrefixlen")
    def max_prefixlen(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        The maximum prefix size that can be allocated from
        the subnetpool. For IPv4 subnetpools, default is 32. For IPv6 subnetpools,
        default is 128. Changing this updates the max prefixlen of the existing
        subnetpool.
        """
        return pulumi.get(self, "max_prefixlen")

    @max_prefixlen.setter
    def max_prefixlen(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "max_prefixlen", value)

    @_builtins.property
    @pulumi.getter(name="minPrefixlen")
    def min_prefixlen(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        The smallest prefix that can be allocated from a
        subnetpool. For IPv4 subnetpools, default is 8. For IPv6 subnetpools, default
        is 64. Changing this updates the min prefixlen of the existing subnetpool.
        """
        return pulumi.get(self, "min_prefixlen")

    @min_prefixlen.setter
    def min_prefixlen(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "min_prefixlen", value)

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The name of the subnetpool. Changing this updates the name of
        the existing subnetpool.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "name", value)

    @_builtins.property
    @pulumi.getter
    def prefixes(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        A list of subnet prefixes to assign to the subnetpool.
        Neutron API merges adjacent prefixes and treats them as a single prefix. Each
        subnet prefix must be unique among all subnet prefixes in all subnetpools that
        are associated with the address scope. Changing this updates the prefixes list
        of the existing subnetpool.
        """
        return pulumi.get(self, "prefixes")

    @prefixes.setter
    def prefixes(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "prefixes", value)

    @_builtins.property
    @pulumi.getter(name="projectId")
    def project_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The owner of the subnetpool. Required if admin wants to
        create a subnetpool for another project. Changing this creates a new subnetpool.
        """
        return pulumi.get(self, "project_id")

    @project_id.setter
    def project_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "project_id", value)

    @_builtins.property
    @pulumi.getter
    def region(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The region in which to obtain the V2 Networking client.
        A Networking client is needed to create a Neutron subnetpool. If omitted, the
        `region` argument of the provider is used. Changing this creates a new
        subnetpool.
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "region", value)

    @_builtins.property
    @pulumi.getter(name="revisionNumber")
    def revision_number(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        The revision number of the subnetpool.
        """
        return pulumi.get(self, "revision_number")

    @revision_number.setter
    def revision_number(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "revision_number", value)

    @_builtins.property
    @pulumi.getter
    def shared(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        Indicates whether this subnetpool is shared across
        all projects. Changing this updates the shared status of the existing
        subnetpool.
        """
        return pulumi.get(self, "shared")

    @shared.setter
    def shared(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "shared", value)

    @_builtins.property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        A set of string tags for the subnetpool.
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "tags", value)

    @_builtins.property
    @pulumi.getter(name="updatedAt")
    def updated_at(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The time at which subnetpool was created.
        """
        return pulumi.get(self, "updated_at")

    @updated_at.setter
    def updated_at(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "updated_at", value)

    @_builtins.property
    @pulumi.getter(name="valueSpecs")
    def value_specs(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[_builtins.str]]]]:
        """
        Map of additional options.
        """
        return pulumi.get(self, "value_specs")

    @value_specs.setter
    def value_specs(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "value_specs", value)


@pulumi.type_token("openstack:networking/subnetPool:SubnetPool")
class SubnetPool(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 address_scope_id: Optional[pulumi.Input[_builtins.str]] = None,
                 default_prefixlen: Optional[pulumi.Input[_builtins.int]] = None,
                 default_quota: Optional[pulumi.Input[_builtins.int]] = None,
                 description: Optional[pulumi.Input[_builtins.str]] = None,
                 ip_version: Optional[pulumi.Input[_builtins.int]] = None,
                 is_default: Optional[pulumi.Input[_builtins.bool]] = None,
                 max_prefixlen: Optional[pulumi.Input[_builtins.int]] = None,
                 min_prefixlen: Optional[pulumi.Input[_builtins.int]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 prefixes: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 project_id: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 shared: Optional[pulumi.Input[_builtins.bool]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 value_specs: Optional[pulumi.Input[Mapping[str, pulumi.Input[_builtins.str]]]] = None,
                 __props__=None):
        """
        Manages a V2 Neutron subnetpool resource within OpenStack.

        ## Example Usage

        ### Create a Subnet Pool

        ```python
        import pulumi
        import pulumi_openstack as openstack

        subnetpool1 = openstack.networking.SubnetPool("subnetpool_1",
            name="subnetpool_1",
            ip_version=6,
            prefixes=[
                "fdf7:b13d:dead:beef::/64",
                "fd65:86cc:a334:39b7::/64",
            ])
        ```

        ### Create a Subnet from a Subnet Pool

        ```python
        import pulumi
        import pulumi_openstack as openstack

        network1 = openstack.networking.Network("network_1",
            name="network_1",
            admin_state_up=True)
        subnetpool1 = openstack.networking.SubnetPool("subnetpool_1",
            name="subnetpool_1",
            prefixes=["10.11.12.0/24"])
        subnet1 = openstack.networking.Subnet("subnet_1",
            name="subnet_1",
            cidr="10.11.12.0/25",
            network_id=network1.id,
            subnetpool_id=subnetpool1.id)
        ```

        ## Import

        Subnetpools can be imported using the `id`, e.g.

        ```sh
        $ pulumi import openstack:networking/subnetPool:SubnetPool subnetpool_1 832cb7f3-59fe-40cf-8f64-8350ffc03272
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] address_scope_id: The Neutron address scope to assign to the
               subnetpool. Changing this updates the address scope id of the existing
               subnetpool.
        :param pulumi.Input[_builtins.int] default_prefixlen: The size of the prefix to allocate when the cidr
               or prefixlen attributes are omitted when you create the subnet. Defaults to the
               MinPrefixLen. Changing this updates the default prefixlen of the existing
               subnetpool.
        :param pulumi.Input[_builtins.int] default_quota: The per-project quota on the prefix space that can be
               allocated from the subnetpool for project subnets. Changing this updates the
               default quota of the existing subnetpool.
        :param pulumi.Input[_builtins.str] description: The human-readable description for the subnetpool.
               Changing this updates the description of the existing subnetpool.
        :param pulumi.Input[_builtins.int] ip_version: The IP protocol version.
        :param pulumi.Input[_builtins.bool] is_default: Indicates whether the subnetpool is default
               subnetpool or not. Changing this updates the default status of the existing
               subnetpool.
        :param pulumi.Input[_builtins.int] max_prefixlen: The maximum prefix size that can be allocated from
               the subnetpool. For IPv4 subnetpools, default is 32. For IPv6 subnetpools,
               default is 128. Changing this updates the max prefixlen of the existing
               subnetpool.
        :param pulumi.Input[_builtins.int] min_prefixlen: The smallest prefix that can be allocated from a
               subnetpool. For IPv4 subnetpools, default is 8. For IPv6 subnetpools, default
               is 64. Changing this updates the min prefixlen of the existing subnetpool.
        :param pulumi.Input[_builtins.str] name: The name of the subnetpool. Changing this updates the name of
               the existing subnetpool.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] prefixes: A list of subnet prefixes to assign to the subnetpool.
               Neutron API merges adjacent prefixes and treats them as a single prefix. Each
               subnet prefix must be unique among all subnet prefixes in all subnetpools that
               are associated with the address scope. Changing this updates the prefixes list
               of the existing subnetpool.
        :param pulumi.Input[_builtins.str] project_id: The owner of the subnetpool. Required if admin wants to
               create a subnetpool for another project. Changing this creates a new subnetpool.
        :param pulumi.Input[_builtins.str] region: The region in which to obtain the V2 Networking client.
               A Networking client is needed to create a Neutron subnetpool. If omitted, the
               `region` argument of the provider is used. Changing this creates a new
               subnetpool.
        :param pulumi.Input[_builtins.bool] shared: Indicates whether this subnetpool is shared across
               all projects. Changing this updates the shared status of the existing
               subnetpool.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: A set of string tags for the subnetpool.
        :param pulumi.Input[Mapping[str, pulumi.Input[_builtins.str]]] value_specs: Map of additional options.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: SubnetPoolArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Manages a V2 Neutron subnetpool resource within OpenStack.

        ## Example Usage

        ### Create a Subnet Pool

        ```python
        import pulumi
        import pulumi_openstack as openstack

        subnetpool1 = openstack.networking.SubnetPool("subnetpool_1",
            name="subnetpool_1",
            ip_version=6,
            prefixes=[
                "fdf7:b13d:dead:beef::/64",
                "fd65:86cc:a334:39b7::/64",
            ])
        ```

        ### Create a Subnet from a Subnet Pool

        ```python
        import pulumi
        import pulumi_openstack as openstack

        network1 = openstack.networking.Network("network_1",
            name="network_1",
            admin_state_up=True)
        subnetpool1 = openstack.networking.SubnetPool("subnetpool_1",
            name="subnetpool_1",
            prefixes=["10.11.12.0/24"])
        subnet1 = openstack.networking.Subnet("subnet_1",
            name="subnet_1",
            cidr="10.11.12.0/25",
            network_id=network1.id,
            subnetpool_id=subnetpool1.id)
        ```

        ## Import

        Subnetpools can be imported using the `id`, e.g.

        ```sh
        $ pulumi import openstack:networking/subnetPool:SubnetPool subnetpool_1 832cb7f3-59fe-40cf-8f64-8350ffc03272
        ```

        :param str resource_name: The name of the resource.
        :param SubnetPoolArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(SubnetPoolArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 address_scope_id: Optional[pulumi.Input[_builtins.str]] = None,
                 default_prefixlen: Optional[pulumi.Input[_builtins.int]] = None,
                 default_quota: Optional[pulumi.Input[_builtins.int]] = None,
                 description: Optional[pulumi.Input[_builtins.str]] = None,
                 ip_version: Optional[pulumi.Input[_builtins.int]] = None,
                 is_default: Optional[pulumi.Input[_builtins.bool]] = None,
                 max_prefixlen: Optional[pulumi.Input[_builtins.int]] = None,
                 min_prefixlen: Optional[pulumi.Input[_builtins.int]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 prefixes: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 project_id: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 shared: Optional[pulumi.Input[_builtins.bool]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 value_specs: Optional[pulumi.Input[Mapping[str, pulumi.Input[_builtins.str]]]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = SubnetPoolArgs.__new__(SubnetPoolArgs)

            __props__.__dict__["address_scope_id"] = address_scope_id
            __props__.__dict__["default_prefixlen"] = default_prefixlen
            __props__.__dict__["default_quota"] = default_quota
            __props__.__dict__["description"] = description
            __props__.__dict__["ip_version"] = ip_version
            __props__.__dict__["is_default"] = is_default
            __props__.__dict__["max_prefixlen"] = max_prefixlen
            __props__.__dict__["min_prefixlen"] = min_prefixlen
            __props__.__dict__["name"] = name
            if prefixes is None and not opts.urn:
                raise TypeError("Missing required property 'prefixes'")
            __props__.__dict__["prefixes"] = prefixes
            __props__.__dict__["project_id"] = project_id
            __props__.__dict__["region"] = region
            __props__.__dict__["shared"] = shared
            __props__.__dict__["tags"] = tags
            __props__.__dict__["value_specs"] = value_specs
            __props__.__dict__["all_tags"] = None
            __props__.__dict__["created_at"] = None
            __props__.__dict__["revision_number"] = None
            __props__.__dict__["updated_at"] = None
        super(SubnetPool, __self__).__init__(
            'openstack:networking/subnetPool:SubnetPool',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            address_scope_id: Optional[pulumi.Input[_builtins.str]] = None,
            all_tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
            created_at: Optional[pulumi.Input[_builtins.str]] = None,
            default_prefixlen: Optional[pulumi.Input[_builtins.int]] = None,
            default_quota: Optional[pulumi.Input[_builtins.int]] = None,
            description: Optional[pulumi.Input[_builtins.str]] = None,
            ip_version: Optional[pulumi.Input[_builtins.int]] = None,
            is_default: Optional[pulumi.Input[_builtins.bool]] = None,
            max_prefixlen: Optional[pulumi.Input[_builtins.int]] = None,
            min_prefixlen: Optional[pulumi.Input[_builtins.int]] = None,
            name: Optional[pulumi.Input[_builtins.str]] = None,
            prefixes: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
            project_id: Optional[pulumi.Input[_builtins.str]] = None,
            region: Optional[pulumi.Input[_builtins.str]] = None,
            revision_number: Optional[pulumi.Input[_builtins.int]] = None,
            shared: Optional[pulumi.Input[_builtins.bool]] = None,
            tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
            updated_at: Optional[pulumi.Input[_builtins.str]] = None,
            value_specs: Optional[pulumi.Input[Mapping[str, pulumi.Input[_builtins.str]]]] = None) -> 'SubnetPool':
        """
        Get an existing SubnetPool resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] address_scope_id: The Neutron address scope to assign to the
               subnetpool. Changing this updates the address scope id of the existing
               subnetpool.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] all_tags: The collection of tags assigned on the subnetpool, which have been
               explicitly and implicitly added.
        :param pulumi.Input[_builtins.str] created_at: The time at which subnetpool was created.
        :param pulumi.Input[_builtins.int] default_prefixlen: The size of the prefix to allocate when the cidr
               or prefixlen attributes are omitted when you create the subnet. Defaults to the
               MinPrefixLen. Changing this updates the default prefixlen of the existing
               subnetpool.
        :param pulumi.Input[_builtins.int] default_quota: The per-project quota on the prefix space that can be
               allocated from the subnetpool for project subnets. Changing this updates the
               default quota of the existing subnetpool.
        :param pulumi.Input[_builtins.str] description: The human-readable description for the subnetpool.
               Changing this updates the description of the existing subnetpool.
        :param pulumi.Input[_builtins.int] ip_version: The IP protocol version.
        :param pulumi.Input[_builtins.bool] is_default: Indicates whether the subnetpool is default
               subnetpool or not. Changing this updates the default status of the existing
               subnetpool.
        :param pulumi.Input[_builtins.int] max_prefixlen: The maximum prefix size that can be allocated from
               the subnetpool. For IPv4 subnetpools, default is 32. For IPv6 subnetpools,
               default is 128. Changing this updates the max prefixlen of the existing
               subnetpool.
        :param pulumi.Input[_builtins.int] min_prefixlen: The smallest prefix that can be allocated from a
               subnetpool. For IPv4 subnetpools, default is 8. For IPv6 subnetpools, default
               is 64. Changing this updates the min prefixlen of the existing subnetpool.
        :param pulumi.Input[_builtins.str] name: The name of the subnetpool. Changing this updates the name of
               the existing subnetpool.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] prefixes: A list of subnet prefixes to assign to the subnetpool.
               Neutron API merges adjacent prefixes and treats them as a single prefix. Each
               subnet prefix must be unique among all subnet prefixes in all subnetpools that
               are associated with the address scope. Changing this updates the prefixes list
               of the existing subnetpool.
        :param pulumi.Input[_builtins.str] project_id: The owner of the subnetpool. Required if admin wants to
               create a subnetpool for another project. Changing this creates a new subnetpool.
        :param pulumi.Input[_builtins.str] region: The region in which to obtain the V2 Networking client.
               A Networking client is needed to create a Neutron subnetpool. If omitted, the
               `region` argument of the provider is used. Changing this creates a new
               subnetpool.
        :param pulumi.Input[_builtins.int] revision_number: The revision number of the subnetpool.
        :param pulumi.Input[_builtins.bool] shared: Indicates whether this subnetpool is shared across
               all projects. Changing this updates the shared status of the existing
               subnetpool.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: A set of string tags for the subnetpool.
        :param pulumi.Input[_builtins.str] updated_at: The time at which subnetpool was created.
        :param pulumi.Input[Mapping[str, pulumi.Input[_builtins.str]]] value_specs: Map of additional options.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _SubnetPoolState.__new__(_SubnetPoolState)

        __props__.__dict__["address_scope_id"] = address_scope_id
        __props__.__dict__["all_tags"] = all_tags
        __props__.__dict__["created_at"] = created_at
        __props__.__dict__["default_prefixlen"] = default_prefixlen
        __props__.__dict__["default_quota"] = default_quota
        __props__.__dict__["description"] = description
        __props__.__dict__["ip_version"] = ip_version
        __props__.__dict__["is_default"] = is_default
        __props__.__dict__["max_prefixlen"] = max_prefixlen
        __props__.__dict__["min_prefixlen"] = min_prefixlen
        __props__.__dict__["name"] = name
        __props__.__dict__["prefixes"] = prefixes
        __props__.__dict__["project_id"] = project_id
        __props__.__dict__["region"] = region
        __props__.__dict__["revision_number"] = revision_number
        __props__.__dict__["shared"] = shared
        __props__.__dict__["tags"] = tags
        __props__.__dict__["updated_at"] = updated_at
        __props__.__dict__["value_specs"] = value_specs
        return SubnetPool(resource_name, opts=opts, __props__=__props__)

    @_builtins.property
    @pulumi.getter(name="addressScopeId")
    def address_scope_id(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        The Neutron address scope to assign to the
        subnetpool. Changing this updates the address scope id of the existing
        subnetpool.
        """
        return pulumi.get(self, "address_scope_id")

    @_builtins.property
    @pulumi.getter(name="allTags")
    def all_tags(self) -> pulumi.Output[Sequence[_builtins.str]]:
        """
        The collection of tags assigned on the subnetpool, which have been
        explicitly and implicitly added.
        """
        return pulumi.get(self, "all_tags")

    @_builtins.property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> pulumi.Output[_builtins.str]:
        """
        The time at which subnetpool was created.
        """
        return pulumi.get(self, "created_at")

    @_builtins.property
    @pulumi.getter(name="defaultPrefixlen")
    def default_prefixlen(self) -> pulumi.Output[_builtins.int]:
        """
        The size of the prefix to allocate when the cidr
        or prefixlen attributes are omitted when you create the subnet. Defaults to the
        MinPrefixLen. Changing this updates the default prefixlen of the existing
        subnetpool.
        """
        return pulumi.get(self, "default_prefixlen")

    @_builtins.property
    @pulumi.getter(name="defaultQuota")
    def default_quota(self) -> pulumi.Output[Optional[_builtins.int]]:
        """
        The per-project quota on the prefix space that can be
        allocated from the subnetpool for project subnets. Changing this updates the
        default quota of the existing subnetpool.
        """
        return pulumi.get(self, "default_quota")

    @_builtins.property
    @pulumi.getter
    def description(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        The human-readable description for the subnetpool.
        Changing this updates the description of the existing subnetpool.
        """
        return pulumi.get(self, "description")

    @_builtins.property
    @pulumi.getter(name="ipVersion")
    def ip_version(self) -> pulumi.Output[_builtins.int]:
        """
        The IP protocol version.
        """
        return pulumi.get(self, "ip_version")

    @_builtins.property
    @pulumi.getter(name="isDefault")
    def is_default(self) -> pulumi.Output[Optional[_builtins.bool]]:
        """
        Indicates whether the subnetpool is default
        subnetpool or not. Changing this updates the default status of the existing
        subnetpool.
        """
        return pulumi.get(self, "is_default")

    @_builtins.property
    @pulumi.getter(name="maxPrefixlen")
    def max_prefixlen(self) -> pulumi.Output[_builtins.int]:
        """
        The maximum prefix size that can be allocated from
        the subnetpool. For IPv4 subnetpools, default is 32. For IPv6 subnetpools,
        default is 128. Changing this updates the max prefixlen of the existing
        subnetpool.
        """
        return pulumi.get(self, "max_prefixlen")

    @_builtins.property
    @pulumi.getter(name="minPrefixlen")
    def min_prefixlen(self) -> pulumi.Output[_builtins.int]:
        """
        The smallest prefix that can be allocated from a
        subnetpool. For IPv4 subnetpools, default is 8. For IPv6 subnetpools, default
        is 64. Changing this updates the min prefixlen of the existing subnetpool.
        """
        return pulumi.get(self, "min_prefixlen")

    @_builtins.property
    @pulumi.getter
    def name(self) -> pulumi.Output[_builtins.str]:
        """
        The name of the subnetpool. Changing this updates the name of
        the existing subnetpool.
        """
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter
    def prefixes(self) -> pulumi.Output[Sequence[_builtins.str]]:
        """
        A list of subnet prefixes to assign to the subnetpool.
        Neutron API merges adjacent prefixes and treats them as a single prefix. Each
        subnet prefix must be unique among all subnet prefixes in all subnetpools that
        are associated with the address scope. Changing this updates the prefixes list
        of the existing subnetpool.
        """
        return pulumi.get(self, "prefixes")

    @_builtins.property
    @pulumi.getter(name="projectId")
    def project_id(self) -> pulumi.Output[_builtins.str]:
        """
        The owner of the subnetpool. Required if admin wants to
        create a subnetpool for another project. Changing this creates a new subnetpool.
        """
        return pulumi.get(self, "project_id")

    @_builtins.property
    @pulumi.getter
    def region(self) -> pulumi.Output[_builtins.str]:
        """
        The region in which to obtain the V2 Networking client.
        A Networking client is needed to create a Neutron subnetpool. If omitted, the
        `region` argument of the provider is used. Changing this creates a new
        subnetpool.
        """
        return pulumi.get(self, "region")

    @_builtins.property
    @pulumi.getter(name="revisionNumber")
    def revision_number(self) -> pulumi.Output[_builtins.int]:
        """
        The revision number of the subnetpool.
        """
        return pulumi.get(self, "revision_number")

    @_builtins.property
    @pulumi.getter
    def shared(self) -> pulumi.Output[Optional[_builtins.bool]]:
        """
        Indicates whether this subnetpool is shared across
        all projects. Changing this updates the shared status of the existing
        subnetpool.
        """
        return pulumi.get(self, "shared")

    @_builtins.property
    @pulumi.getter
    def tags(self) -> pulumi.Output[Optional[Sequence[_builtins.str]]]:
        """
        A set of string tags for the subnetpool.
        """
        return pulumi.get(self, "tags")

    @_builtins.property
    @pulumi.getter(name="updatedAt")
    def updated_at(self) -> pulumi.Output[_builtins.str]:
        """
        The time at which subnetpool was created.
        """
        return pulumi.get(self, "updated_at")

    @_builtins.property
    @pulumi.getter(name="valueSpecs")
    def value_specs(self) -> pulumi.Output[Optional[Mapping[str, _builtins.str]]]:
        """
        Map of additional options.
        """
        return pulumi.get(self, "value_specs")




# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities
from . import outputs
from ._inputs import *

__all__ = ['VolumeAttachArgs', 'VolumeAttach']

@pulumi.input_type
class VolumeAttachArgs:
    def __init__(__self__, *,
                 instance_id: pulumi.Input[_builtins.str],
                 volume_id: pulumi.Input[_builtins.str],
                 device: Optional[pulumi.Input[_builtins.str]] = None,
                 multiattach: Optional[pulumi.Input[_builtins.bool]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 tag: Optional[pulumi.Input[_builtins.str]] = None,
                 vendor_options: Optional[pulumi.Input['VolumeAttachVendorOptionsArgs']] = None):
        """
        The set of arguments for constructing a VolumeAttach resource.
        :param pulumi.Input[_builtins.str] instance_id: The ID of the Instance to attach the Volume to.
        :param pulumi.Input[_builtins.str] volume_id: The ID of the Volume to attach to an Instance.
        :param pulumi.Input[_builtins.bool] multiattach: Enable attachment of multiattach-capable volumes.
        :param pulumi.Input[_builtins.str] region: The region in which to obtain the V2 Compute client.
               A Compute client is needed to create a volume attachment. If omitted, the
               `region` argument of the provider is used. Changing this creates a
               new volume attachment.
        :param pulumi.Input[_builtins.str] tag: Add a device role tag that is applied to the volume when
               attaching it to the VM. Changing this creates a new volume attachment with
               the new tag. Requires microversion >= 2.49.
        :param pulumi.Input['VolumeAttachVendorOptionsArgs'] vendor_options: Map of additional vendor-specific options.
               Supported options are described below.
        """
        pulumi.set(__self__, "instance_id", instance_id)
        pulumi.set(__self__, "volume_id", volume_id)
        if device is not None:
            pulumi.set(__self__, "device", device)
        if multiattach is not None:
            pulumi.set(__self__, "multiattach", multiattach)
        if region is not None:
            pulumi.set(__self__, "region", region)
        if tag is not None:
            pulumi.set(__self__, "tag", tag)
        if vendor_options is not None:
            pulumi.set(__self__, "vendor_options", vendor_options)

    @_builtins.property
    @pulumi.getter(name="instanceId")
    def instance_id(self) -> pulumi.Input[_builtins.str]:
        """
        The ID of the Instance to attach the Volume to.
        """
        return pulumi.get(self, "instance_id")

    @instance_id.setter
    def instance_id(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "instance_id", value)

    @_builtins.property
    @pulumi.getter(name="volumeId")
    def volume_id(self) -> pulumi.Input[_builtins.str]:
        """
        The ID of the Volume to attach to an Instance.
        """
        return pulumi.get(self, "volume_id")

    @volume_id.setter
    def volume_id(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "volume_id", value)

    @_builtins.property
    @pulumi.getter
    def device(self) -> Optional[pulumi.Input[_builtins.str]]:
        return pulumi.get(self, "device")

    @device.setter
    def device(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "device", value)

    @_builtins.property
    @pulumi.getter
    def multiattach(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        Enable attachment of multiattach-capable volumes.
        """
        return pulumi.get(self, "multiattach")

    @multiattach.setter
    def multiattach(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "multiattach", value)

    @_builtins.property
    @pulumi.getter
    def region(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The region in which to obtain the V2 Compute client.
        A Compute client is needed to create a volume attachment. If omitted, the
        `region` argument of the provider is used. Changing this creates a
        new volume attachment.
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "region", value)

    @_builtins.property
    @pulumi.getter
    def tag(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Add a device role tag that is applied to the volume when
        attaching it to the VM. Changing this creates a new volume attachment with
        the new tag. Requires microversion >= 2.49.
        """
        return pulumi.get(self, "tag")

    @tag.setter
    def tag(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "tag", value)

    @_builtins.property
    @pulumi.getter(name="vendorOptions")
    def vendor_options(self) -> Optional[pulumi.Input['VolumeAttachVendorOptionsArgs']]:
        """
        Map of additional vendor-specific options.
        Supported options are described below.
        """
        return pulumi.get(self, "vendor_options")

    @vendor_options.setter
    def vendor_options(self, value: Optional[pulumi.Input['VolumeAttachVendorOptionsArgs']]):
        pulumi.set(self, "vendor_options", value)


@pulumi.input_type
class _VolumeAttachState:
    def __init__(__self__, *,
                 device: Optional[pulumi.Input[_builtins.str]] = None,
                 instance_id: Optional[pulumi.Input[_builtins.str]] = None,
                 multiattach: Optional[pulumi.Input[_builtins.bool]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 tag: Optional[pulumi.Input[_builtins.str]] = None,
                 vendor_options: Optional[pulumi.Input['VolumeAttachVendorOptionsArgs']] = None,
                 volume_id: Optional[pulumi.Input[_builtins.str]] = None):
        """
        Input properties used for looking up and filtering VolumeAttach resources.
        :param pulumi.Input[_builtins.str] instance_id: The ID of the Instance to attach the Volume to.
        :param pulumi.Input[_builtins.bool] multiattach: Enable attachment of multiattach-capable volumes.
        :param pulumi.Input[_builtins.str] region: The region in which to obtain the V2 Compute client.
               A Compute client is needed to create a volume attachment. If omitted, the
               `region` argument of the provider is used. Changing this creates a
               new volume attachment.
        :param pulumi.Input[_builtins.str] tag: Add a device role tag that is applied to the volume when
               attaching it to the VM. Changing this creates a new volume attachment with
               the new tag. Requires microversion >= 2.49.
        :param pulumi.Input['VolumeAttachVendorOptionsArgs'] vendor_options: Map of additional vendor-specific options.
               Supported options are described below.
        :param pulumi.Input[_builtins.str] volume_id: The ID of the Volume to attach to an Instance.
        """
        if device is not None:
            pulumi.set(__self__, "device", device)
        if instance_id is not None:
            pulumi.set(__self__, "instance_id", instance_id)
        if multiattach is not None:
            pulumi.set(__self__, "multiattach", multiattach)
        if region is not None:
            pulumi.set(__self__, "region", region)
        if tag is not None:
            pulumi.set(__self__, "tag", tag)
        if vendor_options is not None:
            pulumi.set(__self__, "vendor_options", vendor_options)
        if volume_id is not None:
            pulumi.set(__self__, "volume_id", volume_id)

    @_builtins.property
    @pulumi.getter
    def device(self) -> Optional[pulumi.Input[_builtins.str]]:
        return pulumi.get(self, "device")

    @device.setter
    def device(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "device", value)

    @_builtins.property
    @pulumi.getter(name="instanceId")
    def instance_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The ID of the Instance to attach the Volume to.
        """
        return pulumi.get(self, "instance_id")

    @instance_id.setter
    def instance_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "instance_id", value)

    @_builtins.property
    @pulumi.getter
    def multiattach(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        Enable attachment of multiattach-capable volumes.
        """
        return pulumi.get(self, "multiattach")

    @multiattach.setter
    def multiattach(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "multiattach", value)

    @_builtins.property
    @pulumi.getter
    def region(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The region in which to obtain the V2 Compute client.
        A Compute client is needed to create a volume attachment. If omitted, the
        `region` argument of the provider is used. Changing this creates a
        new volume attachment.
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "region", value)

    @_builtins.property
    @pulumi.getter
    def tag(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Add a device role tag that is applied to the volume when
        attaching it to the VM. Changing this creates a new volume attachment with
        the new tag. Requires microversion >= 2.49.
        """
        return pulumi.get(self, "tag")

    @tag.setter
    def tag(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "tag", value)

    @_builtins.property
    @pulumi.getter(name="vendorOptions")
    def vendor_options(self) -> Optional[pulumi.Input['VolumeAttachVendorOptionsArgs']]:
        """
        Map of additional vendor-specific options.
        Supported options are described below.
        """
        return pulumi.get(self, "vendor_options")

    @vendor_options.setter
    def vendor_options(self, value: Optional[pulumi.Input['VolumeAttachVendorOptionsArgs']]):
        pulumi.set(self, "vendor_options", value)

    @_builtins.property
    @pulumi.getter(name="volumeId")
    def volume_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The ID of the Volume to attach to an Instance.
        """
        return pulumi.get(self, "volume_id")

    @volume_id.setter
    def volume_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "volume_id", value)


@pulumi.type_token("openstack:compute/volumeAttach:VolumeAttach")
class VolumeAttach(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 device: Optional[pulumi.Input[_builtins.str]] = None,
                 instance_id: Optional[pulumi.Input[_builtins.str]] = None,
                 multiattach: Optional[pulumi.Input[_builtins.bool]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 tag: Optional[pulumi.Input[_builtins.str]] = None,
                 vendor_options: Optional[pulumi.Input[Union['VolumeAttachVendorOptionsArgs', 'VolumeAttachVendorOptionsArgsDict']]] = None,
                 volume_id: Optional[pulumi.Input[_builtins.str]] = None,
                 __props__=None):
        """
        Attaches a Block Storage Volume to an Instance using the OpenStack
        Compute (Nova) v2 API.

        ## Example Usage

        ### Basic attachment of a single volume to a single instance

        ```python
        import pulumi
        import pulumi_openstack as openstack

        volume1 = openstack.blockstorage.Volume("volume_1",
            name="volume_1",
            size=1)
        instance1 = openstack.compute.Instance("instance_1",
            name="instance_1",
            security_groups=["default"])
        va1 = openstack.compute.VolumeAttach("va_1",
            instance_id=instance1.id,
            volume_id=volume1.id)
        ```

        ### Attaching multiple volumes to a single instance

        ```python
        import pulumi
        import pulumi_openstack as openstack
        import pulumi_std as std

        volumes = []
        for range in [{"value": i} for i in range(0, 2)]:
            volumes.append(openstack.blockstorage.Volume(f"volumes-{range['value']}",
                name=std.format(input="vol-%02d",
                    args=[range["value"] + 1]).result,
                size=1))
        instance1 = openstack.compute.Instance("instance_1",
            name="instance_1",
            security_groups=["default"])
        attachments = []
        for range in [{"value": i} for i in range(0, 2)]:
            attachments.append(openstack.compute.VolumeAttach(f"attachments-{range['value']}",
                instance_id=instance1.id,
                volume_id=volumes[range["value"]].id))
        pulumi.export("volumeDevices", [__item.device for __item in attachments])
        ```

        Note that the above example will not guarantee that the volumes are attached in
        a deterministic manner. The volumes will be attached in a seemingly random
        order.

        If you want to ensure that the volumes are attached in a given order, create
        explicit dependencies between the volumes, such as:

        ```python
        import pulumi
        import pulumi_openstack as openstack
        import pulumi_std as std

        volumes = []
        for range in [{"value": i} for i in range(0, 2)]:
            volumes.append(openstack.blockstorage.Volume(f"volumes-{range['value']}",
                name=std.format(input="vol-%02d",
                    args=[range["value"] + 1]).result,
                size=1))
        instance1 = openstack.compute.Instance("instance_1",
            name="instance_1",
            security_groups=["default"])
        attach1 = openstack.compute.VolumeAttach("attach_1",
            instance_id=instance1.id,
            volume_id=volumes[0].id)
        attach2 = openstack.compute.VolumeAttach("attach_2",
            instance_id=instance1.id,
            volume_id=volumes[1].id,
            opts = pulumi.ResourceOptions(depends_on=[attach1]))
        pulumi.export("volumeDevices", [__item["device"] for __item in attachments])
        ```

        ### Using Multiattach-enabled volumes

        Multiattach Volumes are dependent upon your OpenStack cloud and not all
        clouds support multiattach. Multiattach volumes require a volume_type that has [multiattach enabled](https://docs.openstack.org/cinder/latest/admin/volume-multiattach.html#multiattach-volume-type).

        ```python
        import pulumi
        import pulumi_openstack as openstack

        volume1 = openstack.blockstorage.Volume("volume_1",
            name="volume_1",
            size=1,
            volume_type="multiattach")
        instance1 = openstack.compute.Instance("instance_1",
            name="instance_1",
            security_groups=["default"])
        instance2 = openstack.compute.Instance("instance_2",
            name="instance_2",
            security_groups=["default"])
        va1 = openstack.compute.VolumeAttach("va_1",
            instance_id=instance1.id,
            volume_id=volume1.id,
            multiattach=True)
        va2 = openstack.compute.VolumeAttach("va_2",
            instance_id=instance2.id,
            volume_id=volume1.id,
            multiattach=True,
            opts = pulumi.ResourceOptions(depends_on=[va1]))
        ```

        It is recommended to use `depends_on` for the attach resources
        to enforce the volume attachments to happen one at a time.

        ## Import

        Volume Attachments can be imported using the Instance ID and Volume ID
        separated by a slash, e.g.

        ```sh
        $ pulumi import openstack:compute/volumeAttach:VolumeAttach va_1 89c60255-9bd6-460c-822a-e2b959ede9d2/45670584-225f-46c3-b33e-6707b589b666
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] instance_id: The ID of the Instance to attach the Volume to.
        :param pulumi.Input[_builtins.bool] multiattach: Enable attachment of multiattach-capable volumes.
        :param pulumi.Input[_builtins.str] region: The region in which to obtain the V2 Compute client.
               A Compute client is needed to create a volume attachment. If omitted, the
               `region` argument of the provider is used. Changing this creates a
               new volume attachment.
        :param pulumi.Input[_builtins.str] tag: Add a device role tag that is applied to the volume when
               attaching it to the VM. Changing this creates a new volume attachment with
               the new tag. Requires microversion >= 2.49.
        :param pulumi.Input[Union['VolumeAttachVendorOptionsArgs', 'VolumeAttachVendorOptionsArgsDict']] vendor_options: Map of additional vendor-specific options.
               Supported options are described below.
        :param pulumi.Input[_builtins.str] volume_id: The ID of the Volume to attach to an Instance.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: VolumeAttachArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Attaches a Block Storage Volume to an Instance using the OpenStack
        Compute (Nova) v2 API.

        ## Example Usage

        ### Basic attachment of a single volume to a single instance

        ```python
        import pulumi
        import pulumi_openstack as openstack

        volume1 = openstack.blockstorage.Volume("volume_1",
            name="volume_1",
            size=1)
        instance1 = openstack.compute.Instance("instance_1",
            name="instance_1",
            security_groups=["default"])
        va1 = openstack.compute.VolumeAttach("va_1",
            instance_id=instance1.id,
            volume_id=volume1.id)
        ```

        ### Attaching multiple volumes to a single instance

        ```python
        import pulumi
        import pulumi_openstack as openstack
        import pulumi_std as std

        volumes = []
        for range in [{"value": i} for i in range(0, 2)]:
            volumes.append(openstack.blockstorage.Volume(f"volumes-{range['value']}",
                name=std.format(input="vol-%02d",
                    args=[range["value"] + 1]).result,
                size=1))
        instance1 = openstack.compute.Instance("instance_1",
            name="instance_1",
            security_groups=["default"])
        attachments = []
        for range in [{"value": i} for i in range(0, 2)]:
            attachments.append(openstack.compute.VolumeAttach(f"attachments-{range['value']}",
                instance_id=instance1.id,
                volume_id=volumes[range["value"]].id))
        pulumi.export("volumeDevices", [__item.device for __item in attachments])
        ```

        Note that the above example will not guarantee that the volumes are attached in
        a deterministic manner. The volumes will be attached in a seemingly random
        order.

        If you want to ensure that the volumes are attached in a given order, create
        explicit dependencies between the volumes, such as:

        ```python
        import pulumi
        import pulumi_openstack as openstack
        import pulumi_std as std

        volumes = []
        for range in [{"value": i} for i in range(0, 2)]:
            volumes.append(openstack.blockstorage.Volume(f"volumes-{range['value']}",
                name=std.format(input="vol-%02d",
                    args=[range["value"] + 1]).result,
                size=1))
        instance1 = openstack.compute.Instance("instance_1",
            name="instance_1",
            security_groups=["default"])
        attach1 = openstack.compute.VolumeAttach("attach_1",
            instance_id=instance1.id,
            volume_id=volumes[0].id)
        attach2 = openstack.compute.VolumeAttach("attach_2",
            instance_id=instance1.id,
            volume_id=volumes[1].id,
            opts = pulumi.ResourceOptions(depends_on=[attach1]))
        pulumi.export("volumeDevices", [__item["device"] for __item in attachments])
        ```

        ### Using Multiattach-enabled volumes

        Multiattach Volumes are dependent upon your OpenStack cloud and not all
        clouds support multiattach. Multiattach volumes require a volume_type that has [multiattach enabled](https://docs.openstack.org/cinder/latest/admin/volume-multiattach.html#multiattach-volume-type).

        ```python
        import pulumi
        import pulumi_openstack as openstack

        volume1 = openstack.blockstorage.Volume("volume_1",
            name="volume_1",
            size=1,
            volume_type="multiattach")
        instance1 = openstack.compute.Instance("instance_1",
            name="instance_1",
            security_groups=["default"])
        instance2 = openstack.compute.Instance("instance_2",
            name="instance_2",
            security_groups=["default"])
        va1 = openstack.compute.VolumeAttach("va_1",
            instance_id=instance1.id,
            volume_id=volume1.id,
            multiattach=True)
        va2 = openstack.compute.VolumeAttach("va_2",
            instance_id=instance2.id,
            volume_id=volume1.id,
            multiattach=True,
            opts = pulumi.ResourceOptions(depends_on=[va1]))
        ```

        It is recommended to use `depends_on` for the attach resources
        to enforce the volume attachments to happen one at a time.

        ## Import

        Volume Attachments can be imported using the Instance ID and Volume ID
        separated by a slash, e.g.

        ```sh
        $ pulumi import openstack:compute/volumeAttach:VolumeAttach va_1 89c60255-9bd6-460c-822a-e2b959ede9d2/45670584-225f-46c3-b33e-6707b589b666
        ```

        :param str resource_name: The name of the resource.
        :param VolumeAttachArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(VolumeAttachArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 device: Optional[pulumi.Input[_builtins.str]] = None,
                 instance_id: Optional[pulumi.Input[_builtins.str]] = None,
                 multiattach: Optional[pulumi.Input[_builtins.bool]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 tag: Optional[pulumi.Input[_builtins.str]] = None,
                 vendor_options: Optional[pulumi.Input[Union['VolumeAttachVendorOptionsArgs', 'VolumeAttachVendorOptionsArgsDict']]] = None,
                 volume_id: Optional[pulumi.Input[_builtins.str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = VolumeAttachArgs.__new__(VolumeAttachArgs)

            __props__.__dict__["device"] = device
            if instance_id is None and not opts.urn:
                raise TypeError("Missing required property 'instance_id'")
            __props__.__dict__["instance_id"] = instance_id
            __props__.__dict__["multiattach"] = multiattach
            __props__.__dict__["region"] = region
            __props__.__dict__["tag"] = tag
            __props__.__dict__["vendor_options"] = vendor_options
            if volume_id is None and not opts.urn:
                raise TypeError("Missing required property 'volume_id'")
            __props__.__dict__["volume_id"] = volume_id
        super(VolumeAttach, __self__).__init__(
            'openstack:compute/volumeAttach:VolumeAttach',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            device: Optional[pulumi.Input[_builtins.str]] = None,
            instance_id: Optional[pulumi.Input[_builtins.str]] = None,
            multiattach: Optional[pulumi.Input[_builtins.bool]] = None,
            region: Optional[pulumi.Input[_builtins.str]] = None,
            tag: Optional[pulumi.Input[_builtins.str]] = None,
            vendor_options: Optional[pulumi.Input[Union['VolumeAttachVendorOptionsArgs', 'VolumeAttachVendorOptionsArgsDict']]] = None,
            volume_id: Optional[pulumi.Input[_builtins.str]] = None) -> 'VolumeAttach':
        """
        Get an existing VolumeAttach resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] instance_id: The ID of the Instance to attach the Volume to.
        :param pulumi.Input[_builtins.bool] multiattach: Enable attachment of multiattach-capable volumes.
        :param pulumi.Input[_builtins.str] region: The region in which to obtain the V2 Compute client.
               A Compute client is needed to create a volume attachment. If omitted, the
               `region` argument of the provider is used. Changing this creates a
               new volume attachment.
        :param pulumi.Input[_builtins.str] tag: Add a device role tag that is applied to the volume when
               attaching it to the VM. Changing this creates a new volume attachment with
               the new tag. Requires microversion >= 2.49.
        :param pulumi.Input[Union['VolumeAttachVendorOptionsArgs', 'VolumeAttachVendorOptionsArgsDict']] vendor_options: Map of additional vendor-specific options.
               Supported options are described below.
        :param pulumi.Input[_builtins.str] volume_id: The ID of the Volume to attach to an Instance.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _VolumeAttachState.__new__(_VolumeAttachState)

        __props__.__dict__["device"] = device
        __props__.__dict__["instance_id"] = instance_id
        __props__.__dict__["multiattach"] = multiattach
        __props__.__dict__["region"] = region
        __props__.__dict__["tag"] = tag
        __props__.__dict__["vendor_options"] = vendor_options
        __props__.__dict__["volume_id"] = volume_id
        return VolumeAttach(resource_name, opts=opts, __props__=__props__)

    @_builtins.property
    @pulumi.getter
    def device(self) -> pulumi.Output[_builtins.str]:
        return pulumi.get(self, "device")

    @_builtins.property
    @pulumi.getter(name="instanceId")
    def instance_id(self) -> pulumi.Output[_builtins.str]:
        """
        The ID of the Instance to attach the Volume to.
        """
        return pulumi.get(self, "instance_id")

    @_builtins.property
    @pulumi.getter
    def multiattach(self) -> pulumi.Output[Optional[_builtins.bool]]:
        """
        Enable attachment of multiattach-capable volumes.
        """
        return pulumi.get(self, "multiattach")

    @_builtins.property
    @pulumi.getter
    def region(self) -> pulumi.Output[_builtins.str]:
        """
        The region in which to obtain the V2 Compute client.
        A Compute client is needed to create a volume attachment. If omitted, the
        `region` argument of the provider is used. Changing this creates a
        new volume attachment.
        """
        return pulumi.get(self, "region")

    @_builtins.property
    @pulumi.getter
    def tag(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        Add a device role tag that is applied to the volume when
        attaching it to the VM. Changing this creates a new volume attachment with
        the new tag. Requires microversion >= 2.49.
        """
        return pulumi.get(self, "tag")

    @_builtins.property
    @pulumi.getter(name="vendorOptions")
    def vendor_options(self) -> pulumi.Output[Optional['outputs.VolumeAttachVendorOptions']]:
        """
        Map of additional vendor-specific options.
        Supported options are described below.
        """
        return pulumi.get(self, "vendor_options")

    @_builtins.property
    @pulumi.getter(name="volumeId")
    def volume_id(self) -> pulumi.Output[_builtins.str]:
        """
        The ID of the Volume to attach to an Instance.
        """
        return pulumi.get(self, "volume_id")




# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities
from . import outputs

__all__ = [
    'GetLoadbalancerV2Result',
    'AwaitableGetLoadbalancerV2Result',
    'get_loadbalancer_v2',
    'get_loadbalancer_v2_output',
]

@pulumi.output_type
class GetLoadbalancerV2Result:
    """
    A collection of values returned by getLoadbalancerV2.
    """
    def __init__(__self__, additional_vips=None, admin_state_up=None, availability_zone=None, description=None, flavor_id=None, id=None, listeners=None, loadbalancer_id=None, loadbalancer_provider=None, name=None, operating_status=None, pools=None, project_id=None, provisioning_status=None, region=None, tags=None, tags_anies=None, tags_not_anies=None, tags_nots=None, vip_address=None, vip_network_id=None, vip_port_id=None, vip_qos_policy_id=None, vip_subnet_id=None):
        if additional_vips and not isinstance(additional_vips, list):
            raise TypeError("Expected argument 'additional_vips' to be a list")
        pulumi.set(__self__, "additional_vips", additional_vips)
        if admin_state_up and not isinstance(admin_state_up, bool):
            raise TypeError("Expected argument 'admin_state_up' to be a bool")
        pulumi.set(__self__, "admin_state_up", admin_state_up)
        if availability_zone and not isinstance(availability_zone, str):
            raise TypeError("Expected argument 'availability_zone' to be a str")
        pulumi.set(__self__, "availability_zone", availability_zone)
        if description and not isinstance(description, str):
            raise TypeError("Expected argument 'description' to be a str")
        pulumi.set(__self__, "description", description)
        if flavor_id and not isinstance(flavor_id, str):
            raise TypeError("Expected argument 'flavor_id' to be a str")
        pulumi.set(__self__, "flavor_id", flavor_id)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if listeners and not isinstance(listeners, list):
            raise TypeError("Expected argument 'listeners' to be a list")
        pulumi.set(__self__, "listeners", listeners)
        if loadbalancer_id and not isinstance(loadbalancer_id, str):
            raise TypeError("Expected argument 'loadbalancer_id' to be a str")
        pulumi.set(__self__, "loadbalancer_id", loadbalancer_id)
        if loadbalancer_provider and not isinstance(loadbalancer_provider, str):
            raise TypeError("Expected argument 'loadbalancer_provider' to be a str")
        pulumi.set(__self__, "loadbalancer_provider", loadbalancer_provider)
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        pulumi.set(__self__, "name", name)
        if operating_status and not isinstance(operating_status, str):
            raise TypeError("Expected argument 'operating_status' to be a str")
        pulumi.set(__self__, "operating_status", operating_status)
        if pools and not isinstance(pools, list):
            raise TypeError("Expected argument 'pools' to be a list")
        pulumi.set(__self__, "pools", pools)
        if project_id and not isinstance(project_id, str):
            raise TypeError("Expected argument 'project_id' to be a str")
        pulumi.set(__self__, "project_id", project_id)
        if provisioning_status and not isinstance(provisioning_status, str):
            raise TypeError("Expected argument 'provisioning_status' to be a str")
        pulumi.set(__self__, "provisioning_status", provisioning_status)
        if region and not isinstance(region, str):
            raise TypeError("Expected argument 'region' to be a str")
        pulumi.set(__self__, "region", region)
        if tags and not isinstance(tags, list):
            raise TypeError("Expected argument 'tags' to be a list")
        pulumi.set(__self__, "tags", tags)
        if tags_anies and not isinstance(tags_anies, list):
            raise TypeError("Expected argument 'tags_anies' to be a list")
        pulumi.set(__self__, "tags_anies", tags_anies)
        if tags_not_anies and not isinstance(tags_not_anies, list):
            raise TypeError("Expected argument 'tags_not_anies' to be a list")
        pulumi.set(__self__, "tags_not_anies", tags_not_anies)
        if tags_nots and not isinstance(tags_nots, list):
            raise TypeError("Expected argument 'tags_nots' to be a list")
        pulumi.set(__self__, "tags_nots", tags_nots)
        if vip_address and not isinstance(vip_address, str):
            raise TypeError("Expected argument 'vip_address' to be a str")
        pulumi.set(__self__, "vip_address", vip_address)
        if vip_network_id and not isinstance(vip_network_id, str):
            raise TypeError("Expected argument 'vip_network_id' to be a str")
        pulumi.set(__self__, "vip_network_id", vip_network_id)
        if vip_port_id and not isinstance(vip_port_id, str):
            raise TypeError("Expected argument 'vip_port_id' to be a str")
        pulumi.set(__self__, "vip_port_id", vip_port_id)
        if vip_qos_policy_id and not isinstance(vip_qos_policy_id, str):
            raise TypeError("Expected argument 'vip_qos_policy_id' to be a str")
        pulumi.set(__self__, "vip_qos_policy_id", vip_qos_policy_id)
        if vip_subnet_id and not isinstance(vip_subnet_id, str):
            raise TypeError("Expected argument 'vip_subnet_id' to be a str")
        pulumi.set(__self__, "vip_subnet_id", vip_subnet_id)

    @_builtins.property
    @pulumi.getter(name="additionalVips")
    def additional_vips(self) -> Sequence[_builtins.str]:
        """
        A list of additional VIP IP addresses associated with
        the loadbalancer.
        """
        return pulumi.get(self, "additional_vips")

    @_builtins.property
    @pulumi.getter(name="adminStateUp")
    def admin_state_up(self) -> _builtins.bool:
        """
        The administrative state of the loadbalancer (true/false).
        """
        return pulumi.get(self, "admin_state_up")

    @_builtins.property
    @pulumi.getter(name="availabilityZone")
    def availability_zone(self) -> _builtins.str:
        """
        The availability zone of the loadbalancer.
        """
        return pulumi.get(self, "availability_zone")

    @_builtins.property
    @pulumi.getter
    def description(self) -> _builtins.str:
        """
        The description of the loadbalancer.
        """
        return pulumi.get(self, "description")

    @_builtins.property
    @pulumi.getter(name="flavorId")
    def flavor_id(self) -> _builtins.str:
        """
        The flavor ID used by the loadbalancer.
        """
        return pulumi.get(self, "flavor_id")

    @_builtins.property
    @pulumi.getter
    def id(self) -> _builtins.str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter
    def listeners(self) -> Sequence['outputs.GetLoadbalancerV2ListenerResult']:
        """
        A list of listener IDs (UUIDs) associated with the loadbalancer.
        """
        return pulumi.get(self, "listeners")

    @_builtins.property
    @pulumi.getter(name="loadbalancerId")
    def loadbalancer_id(self) -> _builtins.str:
        return pulumi.get(self, "loadbalancer_id")

    @_builtins.property
    @pulumi.getter(name="loadbalancerProvider")
    def loadbalancer_provider(self) -> _builtins.str:
        """
        The loadbalancer driver/provider used by Octavia
        (for example, `amphora`).
        """
        return pulumi.get(self, "loadbalancer_provider")

    @_builtins.property
    @pulumi.getter
    def name(self) -> _builtins.str:
        """
        The name of the loadbalancer.
        """
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter(name="operatingStatus")
    def operating_status(self) -> _builtins.str:
        """
        The operating status of the loadbalancer.
        """
        return pulumi.get(self, "operating_status")

    @_builtins.property
    @pulumi.getter
    def pools(self) -> Sequence['outputs.GetLoadbalancerV2PoolResult']:
        """
        A list of pool IDs (UUIDs) associated with the loadbalancer.
        """
        return pulumi.get(self, "pools")

    @_builtins.property
    @pulumi.getter(name="projectId")
    def project_id(self) -> _builtins.str:
        """
        The owner (project/tenant) ID of the loadbalancer.
        """
        return pulumi.get(self, "project_id")

    @_builtins.property
    @pulumi.getter(name="provisioningStatus")
    def provisioning_status(self) -> _builtins.str:
        """
        The provisioning status of the loadbalancer.
        """
        return pulumi.get(self, "provisioning_status")

    @_builtins.property
    @pulumi.getter
    def region(self) -> _builtins.str:
        return pulumi.get(self, "region")

    @_builtins.property
    @pulumi.getter
    def tags(self) -> Sequence[_builtins.str]:
        """
        A set of tags applied to the loadbalancer.
        """
        return pulumi.get(self, "tags")

    @_builtins.property
    @pulumi.getter(name="tagsAnies")
    def tags_anies(self) -> Optional[Sequence[_builtins.str]]:
        return pulumi.get(self, "tags_anies")

    @_builtins.property
    @pulumi.getter(name="tagsNotAnies")
    def tags_not_anies(self) -> Optional[Sequence[_builtins.str]]:
        return pulumi.get(self, "tags_not_anies")

    @_builtins.property
    @pulumi.getter(name="tagsNots")
    def tags_nots(self) -> Optional[Sequence[_builtins.str]]:
        return pulumi.get(self, "tags_nots")

    @_builtins.property
    @pulumi.getter(name="vipAddress")
    def vip_address(self) -> _builtins.str:
        """
        The IP address of the loadbalancer’s virtual IP (VIP).
        """
        return pulumi.get(self, "vip_address")

    @_builtins.property
    @pulumi.getter(name="vipNetworkId")
    def vip_network_id(self) -> _builtins.str:
        """
        The network ID associated with the VIP.
        """
        return pulumi.get(self, "vip_network_id")

    @_builtins.property
    @pulumi.getter(name="vipPortId")
    def vip_port_id(self) -> _builtins.str:
        """
        The port ID associated with the VIP.
        """
        return pulumi.get(self, "vip_port_id")

    @_builtins.property
    @pulumi.getter(name="vipQosPolicyId")
    def vip_qos_policy_id(self) -> _builtins.str:
        """
        The QoS policy ID associated with the VIP, if any.
        """
        return pulumi.get(self, "vip_qos_policy_id")

    @_builtins.property
    @pulumi.getter(name="vipSubnetId")
    def vip_subnet_id(self) -> _builtins.str:
        """
        The subnet ID associated with the VIP.
        """
        return pulumi.get(self, "vip_subnet_id")


class AwaitableGetLoadbalancerV2Result(GetLoadbalancerV2Result):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetLoadbalancerV2Result(
            additional_vips=self.additional_vips,
            admin_state_up=self.admin_state_up,
            availability_zone=self.availability_zone,
            description=self.description,
            flavor_id=self.flavor_id,
            id=self.id,
            listeners=self.listeners,
            loadbalancer_id=self.loadbalancer_id,
            loadbalancer_provider=self.loadbalancer_provider,
            name=self.name,
            operating_status=self.operating_status,
            pools=self.pools,
            project_id=self.project_id,
            provisioning_status=self.provisioning_status,
            region=self.region,
            tags=self.tags,
            tags_anies=self.tags_anies,
            tags_not_anies=self.tags_not_anies,
            tags_nots=self.tags_nots,
            vip_address=self.vip_address,
            vip_network_id=self.vip_network_id,
            vip_port_id=self.vip_port_id,
            vip_qos_policy_id=self.vip_qos_policy_id,
            vip_subnet_id=self.vip_subnet_id)


def get_loadbalancer_v2(description: Optional[_builtins.str] = None,
                        loadbalancer_id: Optional[_builtins.str] = None,
                        name: Optional[_builtins.str] = None,
                        region: Optional[_builtins.str] = None,
                        tags: Optional[Sequence[_builtins.str]] = None,
                        tags_anies: Optional[Sequence[_builtins.str]] = None,
                        tags_not_anies: Optional[Sequence[_builtins.str]] = None,
                        tags_nots: Optional[Sequence[_builtins.str]] = None,
                        vip_address: Optional[_builtins.str] = None,
                        opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetLoadbalancerV2Result:
    """
    Use this data source to get the ID of an OpenStack Load Balancer.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_openstack as openstack

    loadbalancer1 = openstack.loadbalancer.get_loadbalancer_v2(name="loadbalancer_1")
    ```


    :param _builtins.str description: The human-readable description for the loadbalancer.
    :param _builtins.str loadbalancer_id: The ID of the loadbalancer. Exactly one of
           `name`, `loadbalancer_id` is required to be set.
    :param _builtins.str name: The name of the loadbalancer. Exactly one of `name`,
           `loadbalancer_id` is required to be set.
    :param _builtins.str region: The region in which to obtain the V2 Load Balancer client.
           If omitted, the `region` argument of the provider is used.
    :param Sequence[_builtins.str] tags: A set of tags applied to the loadbalancer. The load balancer
           will be returned if it has all of the specified tags.
    :param Sequence[_builtins.str] tags_anies: A set of tags. The load balancer will be returned if
           it has at least one of the specified tags.
    :param Sequence[_builtins.str] tags_not_anies: A set of tags. The load balancer will be returned
           if it does not have any of the specified tags.
    :param Sequence[_builtins.str] tags_nots: A set of tags. The load balancer will be returned if
           it does not have all of the specified tags.
    :param _builtins.str vip_address: The IP address of the loadbalancer's virtual IP (VIP).
    """
    __args__ = dict()
    __args__['description'] = description
    __args__['loadbalancerId'] = loadbalancer_id
    __args__['name'] = name
    __args__['region'] = region
    __args__['tags'] = tags
    __args__['tagsAnies'] = tags_anies
    __args__['tagsNotAnies'] = tags_not_anies
    __args__['tagsNots'] = tags_nots
    __args__['vipAddress'] = vip_address
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('openstack:loadbalancer/getLoadbalancerV2:getLoadbalancerV2', __args__, opts=opts, typ=GetLoadbalancerV2Result).value

    return AwaitableGetLoadbalancerV2Result(
        additional_vips=pulumi.get(__ret__, 'additional_vips'),
        admin_state_up=pulumi.get(__ret__, 'admin_state_up'),
        availability_zone=pulumi.get(__ret__, 'availability_zone'),
        description=pulumi.get(__ret__, 'description'),
        flavor_id=pulumi.get(__ret__, 'flavor_id'),
        id=pulumi.get(__ret__, 'id'),
        listeners=pulumi.get(__ret__, 'listeners'),
        loadbalancer_id=pulumi.get(__ret__, 'loadbalancer_id'),
        loadbalancer_provider=pulumi.get(__ret__, 'loadbalancer_provider'),
        name=pulumi.get(__ret__, 'name'),
        operating_status=pulumi.get(__ret__, 'operating_status'),
        pools=pulumi.get(__ret__, 'pools'),
        project_id=pulumi.get(__ret__, 'project_id'),
        provisioning_status=pulumi.get(__ret__, 'provisioning_status'),
        region=pulumi.get(__ret__, 'region'),
        tags=pulumi.get(__ret__, 'tags'),
        tags_anies=pulumi.get(__ret__, 'tags_anies'),
        tags_not_anies=pulumi.get(__ret__, 'tags_not_anies'),
        tags_nots=pulumi.get(__ret__, 'tags_nots'),
        vip_address=pulumi.get(__ret__, 'vip_address'),
        vip_network_id=pulumi.get(__ret__, 'vip_network_id'),
        vip_port_id=pulumi.get(__ret__, 'vip_port_id'),
        vip_qos_policy_id=pulumi.get(__ret__, 'vip_qos_policy_id'),
        vip_subnet_id=pulumi.get(__ret__, 'vip_subnet_id'))
def get_loadbalancer_v2_output(description: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                               loadbalancer_id: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                               name: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                               region: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                               tags: Optional[pulumi.Input[Optional[Sequence[_builtins.str]]]] = None,
                               tags_anies: Optional[pulumi.Input[Optional[Sequence[_builtins.str]]]] = None,
                               tags_not_anies: Optional[pulumi.Input[Optional[Sequence[_builtins.str]]]] = None,
                               tags_nots: Optional[pulumi.Input[Optional[Sequence[_builtins.str]]]] = None,
                               vip_address: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                               opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetLoadbalancerV2Result]:
    """
    Use this data source to get the ID of an OpenStack Load Balancer.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_openstack as openstack

    loadbalancer1 = openstack.loadbalancer.get_loadbalancer_v2(name="loadbalancer_1")
    ```


    :param _builtins.str description: The human-readable description for the loadbalancer.
    :param _builtins.str loadbalancer_id: The ID of the loadbalancer. Exactly one of
           `name`, `loadbalancer_id` is required to be set.
    :param _builtins.str name: The name of the loadbalancer. Exactly one of `name`,
           `loadbalancer_id` is required to be set.
    :param _builtins.str region: The region in which to obtain the V2 Load Balancer client.
           If omitted, the `region` argument of the provider is used.
    :param Sequence[_builtins.str] tags: A set of tags applied to the loadbalancer. The load balancer
           will be returned if it has all of the specified tags.
    :param Sequence[_builtins.str] tags_anies: A set of tags. The load balancer will be returned if
           it has at least one of the specified tags.
    :param Sequence[_builtins.str] tags_not_anies: A set of tags. The load balancer will be returned
           if it does not have any of the specified tags.
    :param Sequence[_builtins.str] tags_nots: A set of tags. The load balancer will be returned if
           it does not have all of the specified tags.
    :param _builtins.str vip_address: The IP address of the loadbalancer's virtual IP (VIP).
    """
    __args__ = dict()
    __args__['description'] = description
    __args__['loadbalancerId'] = loadbalancer_id
    __args__['name'] = name
    __args__['region'] = region
    __args__['tags'] = tags
    __args__['tagsAnies'] = tags_anies
    __args__['tagsNotAnies'] = tags_not_anies
    __args__['tagsNots'] = tags_nots
    __args__['vipAddress'] = vip_address
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('openstack:loadbalancer/getLoadbalancerV2:getLoadbalancerV2', __args__, opts=opts, typ=GetLoadbalancerV2Result)
    return __ret__.apply(lambda __response__: GetLoadbalancerV2Result(
        additional_vips=pulumi.get(__response__, 'additional_vips'),
        admin_state_up=pulumi.get(__response__, 'admin_state_up'),
        availability_zone=pulumi.get(__response__, 'availability_zone'),
        description=pulumi.get(__response__, 'description'),
        flavor_id=pulumi.get(__response__, 'flavor_id'),
        id=pulumi.get(__response__, 'id'),
        listeners=pulumi.get(__response__, 'listeners'),
        loadbalancer_id=pulumi.get(__response__, 'loadbalancer_id'),
        loadbalancer_provider=pulumi.get(__response__, 'loadbalancer_provider'),
        name=pulumi.get(__response__, 'name'),
        operating_status=pulumi.get(__response__, 'operating_status'),
        pools=pulumi.get(__response__, 'pools'),
        project_id=pulumi.get(__response__, 'project_id'),
        provisioning_status=pulumi.get(__response__, 'provisioning_status'),
        region=pulumi.get(__response__, 'region'),
        tags=pulumi.get(__response__, 'tags'),
        tags_anies=pulumi.get(__response__, 'tags_anies'),
        tags_not_anies=pulumi.get(__response__, 'tags_not_anies'),
        tags_nots=pulumi.get(__response__, 'tags_nots'),
        vip_address=pulumi.get(__response__, 'vip_address'),
        vip_network_id=pulumi.get(__response__, 'vip_network_id'),
        vip_port_id=pulumi.get(__response__, 'vip_port_id'),
        vip_qos_policy_id=pulumi.get(__response__, 'vip_qos_policy_id'),
        vip_subnet_id=pulumi.get(__response__, 'vip_subnet_id')))



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities

allowReauth: Optional[bool]
"""
If set to `false`, OpenStack authorization won't be perfomed
automatically, if the initial auth token get expired. Defaults to `true`
"""

applicationCredentialId: Optional[str]
"""
Application Credential ID to login with.
"""

applicationCredentialName: Optional[str]
"""
Application Credential name to login with.
"""

applicationCredentialSecret: Optional[str]
"""
Application Credential secret to login with.
"""

authUrl: Optional[str]
"""
The Identity authentication URL.
"""

cacertFile: Optional[str]
"""
A Custom CA certificate.
"""

cert: Optional[str]
"""
A client certificate to authenticate with.
"""

cloud: Optional[str]
"""
An entry in a `clouds.yaml` file to use.
"""

defaultDomain: Optional[str]
"""
The name of the Domain ID to scope to if no other domain is specified. Defaults to `default` (Identity v3).
"""

delayedAuth: Optional[bool]
"""
If set to `false`, OpenStack authorization will be perfomed,
every time the service provider client is called. Defaults to `true`.
"""

disableNoCacheHeader: Optional[bool]
"""
If set to `true`, the HTTP `Cache-Control: no-cache` header will not be added by default to all API requests.
"""

domainId: Optional[str]
"""
The ID of the Domain to scope to (Identity v3).
"""

domainName: Optional[str]
"""
The name of the Domain to scope to (Identity v3).
"""

enableLogging: Optional[bool]
"""
Outputs very verbose logs with all calls made to and responses from OpenStack
"""

endpointOverrides: Optional[str]
"""
A map of services with an endpoint to override what was
from the Keystone catalog
"""

endpointType: Optional[str]

insecure: Optional[bool]
"""
Trust self-signed certificates.
"""

key: Optional[str]
"""
A client private key to authenticate with.
"""

maxRetries: Optional[int]
"""
How many times HTTP connection should be retried until giving up.
"""

password: Optional[str]
"""
Password to login with.
"""

projectDomainId: Optional[str]
"""
The ID of the domain where the proejct resides (Identity v3).
"""

projectDomainName: Optional[str]
"""
The name of the domain where the project resides (Identity v3).
"""

region: Optional[str]
"""
The OpenStack region to connect to.
"""

swauth: Optional[bool]
"""
Use Swift's authentication system instead of Keystone. Only used for
interaction with Swift.
"""

systemScope: Optional[bool]
"""
If set to `true`, system scoped authorization will be enabled. Defaults to `false` (Identity v3).
"""

tenantId: Optional[str]
"""
The ID of the Tenant (Identity v2) or Project (Identity v3)
to login with.
"""

tenantName: Optional[str]
"""
The name of the Tenant (Identity v2) or Project (Identity v3)
to login with.
"""

token: Optional[str]
"""
Authentication token to use as an alternative to username/password.
"""

userDomainId: Optional[str]
"""
The ID of the domain where the user resides (Identity v3).
"""

userDomainName: Optional[str]
"""
The name of the domain where the user resides (Identity v3).
"""

userId: Optional[str]
"""
User ID to login with.
"""

userName: Optional[str]
"""
Username to login with.
"""




# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities

__all__ = ['UserMembershipV3Args', 'UserMembershipV3']

@pulumi.input_type
class UserMembershipV3Args:
    def __init__(__self__, *,
                 group_id: pulumi.Input[_builtins.str],
                 user_id: pulumi.Input[_builtins.str],
                 region: Optional[pulumi.Input[_builtins.str]] = None):
        """
        The set of arguments for constructing a UserMembershipV3 resource.
        :param pulumi.Input[_builtins.str] group_id: The UUID of group to which the user will be added.
               Changing this creates a new user membership.
        :param pulumi.Input[_builtins.str] user_id: The UUID of user to use. Changing this creates a new user membership.
        :param pulumi.Input[_builtins.str] region: The region in which to obtain the V3 Identity client.
               If omitted, the `region` argument of the provider is used.
               Changing this creates a new user membership.
        """
        pulumi.set(__self__, "group_id", group_id)
        pulumi.set(__self__, "user_id", user_id)
        if region is not None:
            pulumi.set(__self__, "region", region)

    @_builtins.property
    @pulumi.getter(name="groupId")
    def group_id(self) -> pulumi.Input[_builtins.str]:
        """
        The UUID of group to which the user will be added.
        Changing this creates a new user membership.
        """
        return pulumi.get(self, "group_id")

    @group_id.setter
    def group_id(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "group_id", value)

    @_builtins.property
    @pulumi.getter(name="userId")
    def user_id(self) -> pulumi.Input[_builtins.str]:
        """
        The UUID of user to use. Changing this creates a new user membership.
        """
        return pulumi.get(self, "user_id")

    @user_id.setter
    def user_id(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "user_id", value)

    @_builtins.property
    @pulumi.getter
    def region(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The region in which to obtain the V3 Identity client.
        If omitted, the `region` argument of the provider is used.
        Changing this creates a new user membership.
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "region", value)


@pulumi.input_type
class _UserMembershipV3State:
    def __init__(__self__, *,
                 group_id: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 user_id: Optional[pulumi.Input[_builtins.str]] = None):
        """
        Input properties used for looking up and filtering UserMembershipV3 resources.
        :param pulumi.Input[_builtins.str] group_id: The UUID of group to which the user will be added.
               Changing this creates a new user membership.
        :param pulumi.Input[_builtins.str] region: The region in which to obtain the V3 Identity client.
               If omitted, the `region` argument of the provider is used.
               Changing this creates a new user membership.
        :param pulumi.Input[_builtins.str] user_id: The UUID of user to use. Changing this creates a new user membership.
        """
        if group_id is not None:
            pulumi.set(__self__, "group_id", group_id)
        if region is not None:
            pulumi.set(__self__, "region", region)
        if user_id is not None:
            pulumi.set(__self__, "user_id", user_id)

    @_builtins.property
    @pulumi.getter(name="groupId")
    def group_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The UUID of group to which the user will be added.
        Changing this creates a new user membership.
        """
        return pulumi.get(self, "group_id")

    @group_id.setter
    def group_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "group_id", value)

    @_builtins.property
    @pulumi.getter
    def region(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The region in which to obtain the V3 Identity client.
        If omitted, the `region` argument of the provider is used.
        Changing this creates a new user membership.
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "region", value)

    @_builtins.property
    @pulumi.getter(name="userId")
    def user_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The UUID of user to use. Changing this creates a new user membership.
        """
        return pulumi.get(self, "user_id")

    @user_id.setter
    def user_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "user_id", value)


@pulumi.type_token("openstack:identity/userMembershipV3:UserMembershipV3")
class UserMembershipV3(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 group_id: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 user_id: Optional[pulumi.Input[_builtins.str]] = None,
                 __props__=None):
        """
        Manages a user membership to group V3 resource within OpenStack.

        > **Note:** You _must_ have admin privileges in your OpenStack cloud to use
        this resource.

        ***

        ## Example Usage

        ```python
        import pulumi
        import pulumi_openstack as openstack

        project1 = openstack.identity.Project("project_1", name="project_1")
        user1 = openstack.identity.User("user_1",
            name="user_1",
            default_project_id=project1.id)
        group1 = openstack.identity.GroupV3("group_1",
            name="group_1",
            description="group 1")
        role1 = openstack.identity.Role("role_1", name="role_1")
        user_membership1 = openstack.identity.UserMembershipV3("user_membership_1",
            user_id=user1.id,
            group_id=group1.id)
        role_assignment1 = openstack.identity.RoleAssignment("role_assignment_1",
            group_id=group1.id,
            project_id=project1.id,
            role_id=role1.id)
        ```

        ## Import

        This resource can be imported by specifying all two arguments, separated
        by a forward slash:

        ```sh
        $ pulumi import openstack:identity/userMembershipV3:UserMembershipV3 user_membership_1 user_id/group_id
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] group_id: The UUID of group to which the user will be added.
               Changing this creates a new user membership.
        :param pulumi.Input[_builtins.str] region: The region in which to obtain the V3 Identity client.
               If omitted, the `region` argument of the provider is used.
               Changing this creates a new user membership.
        :param pulumi.Input[_builtins.str] user_id: The UUID of user to use. Changing this creates a new user membership.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: UserMembershipV3Args,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Manages a user membership to group V3 resource within OpenStack.

        > **Note:** You _must_ have admin privileges in your OpenStack cloud to use
        this resource.

        ***

        ## Example Usage

        ```python
        import pulumi
        import pulumi_openstack as openstack

        project1 = openstack.identity.Project("project_1", name="project_1")
        user1 = openstack.identity.User("user_1",
            name="user_1",
            default_project_id=project1.id)
        group1 = openstack.identity.GroupV3("group_1",
            name="group_1",
            description="group 1")
        role1 = openstack.identity.Role("role_1", name="role_1")
        user_membership1 = openstack.identity.UserMembershipV3("user_membership_1",
            user_id=user1.id,
            group_id=group1.id)
        role_assignment1 = openstack.identity.RoleAssignment("role_assignment_1",
            group_id=group1.id,
            project_id=project1.id,
            role_id=role1.id)
        ```

        ## Import

        This resource can be imported by specifying all two arguments, separated
        by a forward slash:

        ```sh
        $ pulumi import openstack:identity/userMembershipV3:UserMembershipV3 user_membership_1 user_id/group_id
        ```

        :param str resource_name: The name of the resource.
        :param UserMembershipV3Args args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(UserMembershipV3Args, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 group_id: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[_builtins.str]] = None,
                 user_id: Optional[pulumi.Input[_builtins.str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = UserMembershipV3Args.__new__(UserMembershipV3Args)

            if group_id is None and not opts.urn:
                raise TypeError("Missing required property 'group_id'")
            __props__.__dict__["group_id"] = group_id
            __props__.__dict__["region"] = region
            if user_id is None and not opts.urn:
                raise TypeError("Missing required property 'user_id'")
            __props__.__dict__["user_id"] = user_id
        super(UserMembershipV3, __self__).__init__(
            'openstack:identity/userMembershipV3:UserMembershipV3',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            group_id: Optional[pulumi.Input[_builtins.str]] = None,
            region: Optional[pulumi.Input[_builtins.str]] = None,
            user_id: Optional[pulumi.Input[_builtins.str]] = None) -> 'UserMembershipV3':
        """
        Get an existing UserMembershipV3 resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] group_id: The UUID of group to which the user will be added.
               Changing this creates a new user membership.
        :param pulumi.Input[_builtins.str] region: The region in which to obtain the V3 Identity client.
               If omitted, the `region` argument of the provider is used.
               Changing this creates a new user membership.
        :param pulumi.Input[_builtins.str] user_id: The UUID of user to use. Changing this creates a new user membership.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _UserMembershipV3State.__new__(_UserMembershipV3State)

        __props__.__dict__["group_id"] = group_id
        __props__.__dict__["region"] = region
        __props__.__dict__["user_id"] = user_id
        return UserMembershipV3(resource_name, opts=opts, __props__=__props__)

    @_builtins.property
    @pulumi.getter(name="groupId")
    def group_id(self) -> pulumi.Output[_builtins.str]:
        """
        The UUID of group to which the user will be added.
        Changing this creates a new user membership.
        """
        return pulumi.get(self, "group_id")

    @_builtins.property
    @pulumi.getter
    def region(self) -> pulumi.Output[_builtins.str]:
        """
        The region in which to obtain the V3 Identity client.
        If omitted, the `region` argument of the provider is used.
        Changing this creates a new user membership.
        """
        return pulumi.get(self, "region")

    @_builtins.property
    @pulumi.getter(name="userId")
    def user_id(self) -> pulumi.Output[_builtins.str]:
        """
        The UUID of user to use. Changing this creates a new user membership.
        """
        return pulumi.get(self, "user_id")




# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities

__all__ = [
    'GetProjectIdsV3Result',
    'AwaitableGetProjectIdsV3Result',
    'get_project_ids_v3',
    'get_project_ids_v3_output',
]

@pulumi.output_type
class GetProjectIdsV3Result:
    """
    A collection of values returned by getProjectIdsV3.
    """
    def __init__(__self__, description=None, domain_id=None, enabled=None, id=None, ids=None, is_domain=None, name=None, name_regex=None, parent_id=None, region=None, tags=None):
        if description and not isinstance(description, str):
            raise TypeError("Expected argument 'description' to be a str")
        pulumi.set(__self__, "description", description)
        if domain_id and not isinstance(domain_id, str):
            raise TypeError("Expected argument 'domain_id' to be a str")
        pulumi.set(__self__, "domain_id", domain_id)
        if enabled and not isinstance(enabled, bool):
            raise TypeError("Expected argument 'enabled' to be a bool")
        pulumi.set(__self__, "enabled", enabled)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if ids and not isinstance(ids, list):
            raise TypeError("Expected argument 'ids' to be a list")
        pulumi.set(__self__, "ids", ids)
        if is_domain and not isinstance(is_domain, bool):
            raise TypeError("Expected argument 'is_domain' to be a bool")
        pulumi.set(__self__, "is_domain", is_domain)
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        pulumi.set(__self__, "name", name)
        if name_regex and not isinstance(name_regex, str):
            raise TypeError("Expected argument 'name_regex' to be a str")
        pulumi.set(__self__, "name_regex", name_regex)
        if parent_id and not isinstance(parent_id, str):
            raise TypeError("Expected argument 'parent_id' to be a str")
        pulumi.set(__self__, "parent_id", parent_id)
        if region and not isinstance(region, str):
            raise TypeError("Expected argument 'region' to be a str")
        pulumi.set(__self__, "region", region)
        if tags and not isinstance(tags, list):
            raise TypeError("Expected argument 'tags' to be a list")
        pulumi.set(__self__, "tags", tags)

    @_builtins.property
    @pulumi.getter
    def description(self) -> _builtins.str:
        return pulumi.get(self, "description")

    @_builtins.property
    @pulumi.getter(name="domainId")
    def domain_id(self) -> _builtins.str:
        return pulumi.get(self, "domain_id")

    @_builtins.property
    @pulumi.getter
    def enabled(self) -> Optional[_builtins.bool]:
        return pulumi.get(self, "enabled")

    @_builtins.property
    @pulumi.getter
    def id(self) -> _builtins.str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter
    def ids(self) -> Sequence[_builtins.str]:
        return pulumi.get(self, "ids")

    @_builtins.property
    @pulumi.getter(name="isDomain")
    def is_domain(self) -> Optional[_builtins.bool]:
        return pulumi.get(self, "is_domain")

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[_builtins.str]:
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter(name="nameRegex")
    def name_regex(self) -> Optional[_builtins.str]:
        return pulumi.get(self, "name_regex")

    @_builtins.property
    @pulumi.getter(name="parentId")
    def parent_id(self) -> Optional[_builtins.str]:
        return pulumi.get(self, "parent_id")

    @_builtins.property
    @pulumi.getter
    def region(self) -> _builtins.str:
        return pulumi.get(self, "region")

    @_builtins.property
    @pulumi.getter
    def tags(self) -> Optional[Sequence[_builtins.str]]:
        return pulumi.get(self, "tags")


class AwaitableGetProjectIdsV3Result(GetProjectIdsV3Result):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetProjectIdsV3Result(
            description=self.description,
            domain_id=self.domain_id,
            enabled=self.enabled,
            id=self.id,
            ids=self.ids,
            is_domain=self.is_domain,
            name=self.name,
            name_regex=self.name_regex,
            parent_id=self.parent_id,
            region=self.region,
            tags=self.tags)


def get_project_ids_v3(domain_id: Optional[_builtins.str] = None,
                       enabled: Optional[_builtins.bool] = None,
                       is_domain: Optional[_builtins.bool] = None,
                       name: Optional[_builtins.str] = None,
                       name_regex: Optional[_builtins.str] = None,
                       parent_id: Optional[_builtins.str] = None,
                       region: Optional[_builtins.str] = None,
                       tags: Optional[Sequence[_builtins.str]] = None,
                       opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetProjectIdsV3Result:
    """
    Use this data source to get a list of OpenStack Project IDs matching the
    specified criteria.

    > **Note:** You _must_ have domain admin or cloud admin privileges in your OpenStack cloud to use
    this datasource.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_openstack as openstack

    projects = openstack.identity.get_project_ids_v3(name_regex="^prod.*")
    ```


    :param _builtins.str domain_id: The domain projects belongs to.
    :param _builtins.bool enabled: Whether the project is enabled or disabled. Valid
           values are `true` and `false`. Default is `true`.
    :param _builtins.str name: The name of the project.
    :param _builtins.str name_regex: The regular expression of the name of the project.
           Cannot be used simultaneously with `name`. Unlike filtering by `name` the
           `name_regex` filtering does by client on the result of OpenStack search
           query.
    :param _builtins.str parent_id: The parent of the project.
    :param _builtins.str region: The region in which to obtain the V3 Keystone client.
           If omitted, the `region` argument of the provider is used.
    :param Sequence[_builtins.str] tags: Tags for the project.
    """
    __args__ = dict()
    __args__['domainId'] = domain_id
    __args__['enabled'] = enabled
    __args__['isDomain'] = is_domain
    __args__['name'] = name
    __args__['nameRegex'] = name_regex
    __args__['parentId'] = parent_id
    __args__['region'] = region
    __args__['tags'] = tags
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('openstack:identity/getProjectIdsV3:getProjectIdsV3', __args__, opts=opts, typ=GetProjectIdsV3Result).value

    return AwaitableGetProjectIdsV3Result(
        description=pulumi.get(__ret__, 'description'),
        domain_id=pulumi.get(__ret__, 'domain_id'),
        enabled=pulumi.get(__ret__, 'enabled'),
        id=pulumi.get(__ret__, 'id'),
        ids=pulumi.get(__ret__, 'ids'),
        is_domain=pulumi.get(__ret__, 'is_domain'),
        name=pulumi.get(__ret__, 'name'),
        name_regex=pulumi.get(__ret__, 'name_regex'),
        parent_id=pulumi.get(__ret__, 'parent_id'),
        region=pulumi.get(__ret__, 'region'),
        tags=pulumi.get(__ret__, 'tags'))
def get_project_ids_v3_output(domain_id: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                              enabled: Optional[pulumi.Input[Optional[_builtins.bool]]] = None,
                              is_domain: Optional[pulumi.Input[Optional[_builtins.bool]]] = None,
                              name: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                              name_regex: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                              parent_id: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                              region: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                              tags: Optional[pulumi.Input[Optional[Sequence[_builtins.str]]]] = None,
                              opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetProjectIdsV3Result]:
    """
    Use this data source to get a list of OpenStack Project IDs matching the
    specified criteria.

    > **Note:** You _must_ have domain admin or cloud admin privileges in your OpenStack cloud to use
    this datasource.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_openstack as openstack

    projects = openstack.identity.get_project_ids_v3(name_regex="^prod.*")
    ```


    :param _builtins.str domain_id: The domain projects belongs to.
    :param _builtins.bool enabled: Whether the project is enabled or disabled. Valid
           values are `true` and `false`. Default is `true`.
    :param _builtins.str name: The name of the project.
    :param _builtins.str name_regex: The regular expression of the name of the project.
           Cannot be used simultaneously with `name`. Unlike filtering by `name` the
           `name_regex` filtering does by client on the result of OpenStack search
           query.
    :param _builtins.str parent_id: The parent of the project.
    :param _builtins.str region: The region in which to obtain the V3 Keystone client.
           If omitted, the `region` argument of the provider is used.
    :param Sequence[_builtins.str] tags: Tags for the project.
    """
    __args__ = dict()
    __args__['domainId'] = domain_id
    __args__['enabled'] = enabled
    __args__['isDomain'] = is_domain
    __args__['name'] = name
    __args__['nameRegex'] = name_regex
    __args__['parentId'] = parent_id
    __args__['region'] = region
    __args__['tags'] = tags
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('openstack:identity/getProjectIdsV3:getProjectIdsV3', __args__, opts=opts, typ=GetProjectIdsV3Result)
    return __ret__.apply(lambda __response__: GetProjectIdsV3Result(
        description=pulumi.get(__response__, 'description'),
        domain_id=pulumi.get(__response__, 'domain_id'),
        enabled=pulumi.get(__response__, 'enabled'),
        id=pulumi.get(__response__, 'id'),
        ids=pulumi.get(__response__, 'ids'),
        is_domain=pulumi.get(__response__, 'is_domain'),
        name=pulumi.get(__response__, 'name'),
        name_regex=pulumi.get(__response__, 'name_regex'),
        parent_id=pulumi.get(__response__, 'parent_id'),
        region=pulumi.get(__response__, 'region'),
        tags=pulumi.get(__response__, 'tags')))
